@@ -21,6 +21,7 @@
 
 /* TODO: sequencer queue timer selection */
 
+#include "aconfig.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -28,8 +29,8 @@
 #include <getopt.h>
 #include <unistd.h>
 #include <alsa/asoundlib.h>
-#include "aconfig.h"
 #include "version.h"
+#include <alsa/ump_msg.h>
 
 /*
  * 31.25 kbaud, one start bit, eight data bits, two stop bits.
@@ -75,6 +76,7 @@ static int file_offset;		/* current offset in input file */
 static int num_tracks;
 static struct track *tracks;
 static int smpte_timing;
+static int ump_mode;
 
 /* prints an error message to stderr */
 static void errormsg(const char *msg, ...)
@@ -618,7 +620,7 @@ static void handle_big_sysex(snd_seq_event_t *ev)
 	if (length > MIDI_BYTES_PER_SEC)
 		ev->data.ext.len = MIDI_BYTES_PER_SEC;
 	event_size = snd_seq_event_length(ev);
-	if (event_size + 1 > snd_seq_get_output_buffer_size(seq)) {
+	if (event_size + 1 > (ssize_t)snd_seq_get_output_buffer_size(seq)) {
 		err = snd_seq_drain_output(seq);
 		check_snd("drain output", err);
 		err = snd_seq_set_output_buffer_size(seq, event_size + 1);
@@ -639,8 +641,126 @@ static void handle_big_sysex(snd_seq_event_t *ev)
 	ev->data.ext.len = length;
 }
 
+static int fill_legacy_event(struct event* event, snd_seq_event_t *ev)
+{
+	ev->type = event->type;
+	switch (ev->type) {
+	case SND_SEQ_EVENT_NOTEON:
+	case SND_SEQ_EVENT_NOTEOFF:
+	case SND_SEQ_EVENT_KEYPRESS:
+		snd_seq_ev_set_fixed(ev);
+		ev->data.note.channel = event->data.d[0];
+		ev->data.note.note = event->data.d[1];
+		ev->data.note.velocity = event->data.d[2];
+		break;
+	case SND_SEQ_EVENT_CONTROLLER:
+		snd_seq_ev_set_fixed(ev);
+		ev->data.control.channel = event->data.d[0];
+		ev->data.control.param = event->data.d[1];
+		ev->data.control.value = event->data.d[2];
+		break;
+	case SND_SEQ_EVENT_PGMCHANGE:
+	case SND_SEQ_EVENT_CHANPRESS:
+		snd_seq_ev_set_fixed(ev);
+		ev->data.control.channel = event->data.d[0];
+		ev->data.control.value = event->data.d[1];
+		break;
+	case SND_SEQ_EVENT_PITCHBEND:
+		snd_seq_ev_set_fixed(ev);
+		ev->data.control.channel = event->data.d[0];
+		ev->data.control.value = ((event->data.d[1]) |
+					  ((event->data.d[2]) << 7)) - 0x2000;
+		break;
+	case SND_SEQ_EVENT_SYSEX:
+		snd_seq_ev_set_variable(ev, event->data.length, event->sysex);
+		handle_big_sysex(ev);
+		break;
+	default:
+		fatal("Invalid event type %d!", ev->type);
+	}
+	return 0;
+}
+
+static unsigned char to_ump_status(unsigned char ev_type)
+{
+	switch (ev_type) {
+	case SND_SEQ_EVENT_NOTEON:
+		return SND_UMP_MSG_NOTE_ON;
+	case SND_SEQ_EVENT_NOTEOFF:
+		return SND_UMP_MSG_NOTE_OFF;
+	case SND_SEQ_EVENT_KEYPRESS:
+		return SND_UMP_MSG_POLY_PRESSURE;
+	case SND_SEQ_EVENT_CONTROLLER:
+		return SND_UMP_MSG_CONTROL_CHANGE;
+	case SND_SEQ_EVENT_PGMCHANGE:
+		return SND_UMP_MSG_PROGRAM_CHANGE;
+	case SND_SEQ_EVENT_CHANPRESS:
+		return SND_UMP_MSG_CHANNEL_PRESSURE;
+	case SND_SEQ_EVENT_PITCHBEND:
+		return SND_UMP_MSG_PITCHBEND;
+	default:
+		return 0;
+	}
+}
+
+static int fill_ump_event(struct event* event, snd_seq_ump_event_t *ump_ev,
+			  const snd_seq_event_t *ev)
+{
+	snd_ump_msg_midi1_t ump = {};
+	unsigned char status = to_ump_status(event->type);
+
+	memcpy(ump_ev, ev, sizeof(*ev));
+	if (!status)
+		return 0; /* handle as is */
+
+	ump.note_on.type = SND_UMP_MSG_TYPE_MIDI1_CHANNEL_VOICE;
+	switch (event->type) {
+	case SND_SEQ_EVENT_NOTEON:
+		/* correct the note-on with velocity 0 to note-off;
+		 * UMP may handle velocity 0 differently
+		 */
+		if (!ev->data.note.velocity)
+			status = SND_UMP_MSG_NOTE_OFF;
+		/* fallthrough */
+	case SND_SEQ_EVENT_NOTEOFF:
+	case SND_SEQ_EVENT_KEYPRESS:
+		ump.note_on.status = status;
+		ump.note_on.channel = event->data.d[0];
+		ump.note_on.note = event->data.d[1];
+		ump.note_on.velocity = event->data.d[2];
+		break;
+	case SND_SEQ_EVENT_CONTROLLER:
+		ump.control_change.status = status;
+		ump.control_change.channel = event->data.d[0];
+		ump.control_change.index = event->data.d[1];
+		ump.control_change.data = event->data.d[2];
+		break;
+	case SND_SEQ_EVENT_PGMCHANGE:
+		ump.program_change.status = status;
+		ump.program_change.channel = event->data.d[0];
+		ump.program_change.program = event->data.d[1];
+		break;
+	case SND_SEQ_EVENT_CHANPRESS:
+		ump.channel_pressure.status = status;
+		ump.channel_pressure.channel = event->data.d[0];
+		ump.channel_pressure.data = event->data.d[1];
+		break;
+	case SND_SEQ_EVENT_PITCHBEND:
+		ump.pitchbend.status = status;
+		ump.pitchbend.channel = event->data.d[0];
+		ump.pitchbend.data_msb = event->data.d[2];
+		ump.pitchbend.data_lsb = event->data.d[1];
+		break;
+	default:
+		return 0; /* handle as is */
+	}
+	snd_seq_ev_set_ump_data(ump_ev, &ump, sizeof(ump));
+	return 0;
+}
+
 static void play_midi(void)
 {
+	snd_seq_ump_event_t ump_ev;
 	snd_seq_event_t ev;
 	int i, max_tick, err;
 
@@ -675,7 +795,7 @@ static void play_midi(void)
 		for (i = 0; i < num_tracks; ++i) {
 			struct track *track = &tracks[i];
 			struct event *e2 = track->current_event;
-			if (e2 && e2->tick < min_tick) {
+			if (e2 && e2->tick < (unsigned int)min_tick) {
 				min_tick = e2->tick;
 				event = e2;
 				event_track = track;
@@ -688,51 +808,28 @@ static void play_midi(void)
 		event_track->current_event = event->next;
 
 		/* output the event */
-		ev.type = event->type;
 		ev.time.tick = event->tick;
 		ev.dest = ports[event->port];
-		switch (ev.type) {
-		case SND_SEQ_EVENT_NOTEON:
-		case SND_SEQ_EVENT_NOTEOFF:
-		case SND_SEQ_EVENT_KEYPRESS:
-			snd_seq_ev_set_fixed(&ev);
-			ev.data.note.channel = event->data.d[0];
-			ev.data.note.note = event->data.d[1];
-			ev.data.note.velocity = event->data.d[2];
-			break;
-		case SND_SEQ_EVENT_CONTROLLER:
-			snd_seq_ev_set_fixed(&ev);
-			ev.data.control.channel = event->data.d[0];
-			ev.data.control.param = event->data.d[1];
-			ev.data.control.value = event->data.d[2];
-			break;
-		case SND_SEQ_EVENT_PGMCHANGE:
-		case SND_SEQ_EVENT_CHANPRESS:
-			snd_seq_ev_set_fixed(&ev);
-			ev.data.control.channel = event->data.d[0];
-			ev.data.control.value = event->data.d[1];
-			break;
-		case SND_SEQ_EVENT_PITCHBEND:
-			snd_seq_ev_set_fixed(&ev);
-			ev.data.control.channel = event->data.d[0];
-			ev.data.control.value =
-				((event->data.d[1]) |
-				 ((event->data.d[2]) << 7)) - 0x2000;
-			break;
-		case SND_SEQ_EVENT_SYSEX:
-			snd_seq_ev_set_variable(&ev, event->data.length,
-						event->sysex);
-			handle_big_sysex(&ev);
-			break;
-		case SND_SEQ_EVENT_TEMPO:
+		if (event->type == SND_SEQ_EVENT_TEMPO) {
 			snd_seq_ev_set_fixed(&ev);
+			ev.type = event->type;
 			ev.dest.client = SND_SEQ_CLIENT_SYSTEM;
 			ev.dest.port = SND_SEQ_PORT_SYSTEM_TIMER;
 			ev.data.queue.queue = queue;
 			ev.data.queue.param.value = event->data.tempo;
-			break;
-		default:
-			fatal("Invalid event type %d!", ev.type);
+		} else {
+			err = fill_legacy_event(event, &ev);
+			if (err < 0)
+				continue;
+		}
+
+		if (ump_mode) {
+			err = fill_ump_event(event, &ump_ev, &ev);
+			if (err < 0)
+				continue;
+			err = snd_seq_ump_event_output(seq, &ump_ev);
+			check_snd("output event", err);
+			continue;
 		}
 
 		/* this blocks when the output pool has been filled */
@@ -851,6 +948,7 @@ static void usage(const char *argv0)
 		"-V, --version               print current version\n"
 		"-l, --list                  list all possible output ports\n"
 		"-p, --port=client:port,...  set port(s) to play to\n"
+		"-u, --ump=version           UMP output (only version=1 is supported)\n"
 		"-d, --delay=seconds         delay after song ends\n",
 		argv0);
 }
@@ -860,14 +958,17 @@ static void version(void)
 	puts("aplaymidi version " SND_UTIL_VERSION_STR);
 }
 
+#define OPTIONS	"hVlp:d:u:"
+
 int main(int argc, char *argv[])
 {
-	static const char short_options[] = "hVlp:d:";
+	static const char short_options[] = OPTIONS;
 	static const struct option long_options[] = {
 		{"help", 0, NULL, 'h'},
 		{"version", 0, NULL, 'V'},
 		{"list", 0, NULL, 'l'},
 		{"port", 1, NULL, 'p'},
+		{"ump", 1, NULL, 'u'},
 		{"delay", 1, NULL, 'd'},
 		{0}
 	};
@@ -894,12 +995,24 @@ int main(int argc, char *argv[])
 		case 'd':
 			end_delay = atoi(optarg);
 			break;
+		case 'u':
+			ump_mode = atoi(optarg);
+			if (ump_mode < 0 || ump_mode > 1)
+				fatal("Only MIDI 1.0 is supported");
+			break;
 		default:
 			usage(argv[0]);
 			return 1;
 		}
 	}
 
+
+	if (ump_mode) {
+		int err;
+		err = snd_seq_set_client_midi_version(seq, SND_SEQ_CLIENT_UMP_MIDI_1_0);
+		check_snd("set midi version", err);
+	}
+
 	if (do_list) {
 		list_ports();
 	} else {
